/****************************************************************************************/
/*	sxCore.h                                                                    	    	*/
/*                                                                                      */
/*  Author: D.R. Gunter                                                                	*/
/*  Description: Core component declarations for RFSX.																	*/
/*                                                                                      */
/*                                                                                      */
/*  See the accompanying file RealityFactory_Software_License.txt for terms 						*/
/*	on the use of this file. (Under "Reality Factory License")													*/
/*	Software distributed under the License is distributed on an "AS IS"                 */
/*  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See                */
/*  the License for the specific language governing rights and limitations              */
/*  under the License.                                                                  */
/*                                                                                      */
/****************************************************************************************/

#ifndef _SX_CORE_H_
#define _SX_CORE_H_

#include <vector>

namespace RFSX
{

	class sxMHT_init
	{
		
		public:
			sxMHT_init();
			~sxMHT_init();
			
			void AddMHT(void (*)(void));
			void execute();
		
		private:
			std::vector<void (*)(void)> MHT_Inits;
		
	};

	// Class defining skString-to-function pointer hashtable for Method functions
	template <typename T> class sxMHT
	{

		public:
			sxMHT<T>(USize size, void (*)(void));
			
		protected:
			inline bool operator()(T* caller, const skString& m_name, skRValueArray& args, skRValue& r_val, skExecutableContext& ctxt);
			inline void Add(skString m_name, SX_P_FUNC_IT(m,T));
		
		private:
			static void InstallMHT(void (*pfunc)(void));
		
			skTSHashTable<SX_P_FUNC_T(T)> MHT;
		
		friend T;
		
	};

	// Type safety functions
	template<typename T> T* CheckType(skRValue& val, int type);
	extern bool CheckParams(int pmin, int pmax, skRValueArray& args, skRValue::RType types[]);

	void Install(skInterpreter* Interp);
	extern sxMHT_init inits;

} // namespace RFSX

#include "sxMHT.inl"

#endif // _RFSX_H_
