/****************************************************************************************/
/*	sxMath.cpp                                                                      	*/
/*                                                                                      */
/*  Author: D.R. Gunter                                                                	*/
/*  Description: Implementation for RFSX File function library													*/
/*                                                                                      */
/*                                                                                      */
/*  See the accompanying file RealityFactory_Software_License.txt for terms 						*/
/*	on the use of this file. (Under "Reality Factory License")													*/
/*	Software distributed under the License is distributed on an "AS IS"                 */
/*  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See                */
/*  the License for the specific language governing rights and limitations              */
/*  under the License.                                                                  */
/*                                                                                      */
/****************************************************************************************/

#include "sxMath.h"

#include <math.h>

#ifdef SX_IMPL_TYPE
#undef SX_IMPL_TYPE
#endif
#define SX_IMPL_TYPE sxMath

namespace RFSX
{
	
//////////////////////////////////////////////////////////////////////////////////////////////////////////
// sxMath class implementation
//////////////////////////////////////////////////////////////////////////////////////////////////////////
	
SX_IMPL_TYPE::SX_IMPL_TYPE()
{
	
	;
	
}

SX_IMPL_TYPE::~SX_IMPL_TYPE()
{
	;
}

bool SX_IMPL_TYPE::method(const skString& m_name, skRValueArray& args, skRValue& r_val, skExecutableContext& ctxt)
{

	if(h_method(this, m_name, args, r_val, ctxt))
		return true;
	else return skExecutable::method(m_name, args, r_val, ctxt);
	
}
	
	
//////////////////////////////////////////////////////////////////////////////////////////////////////////
// sxMath method functions
//////////////////////////////////////////////////////////////////////////////////////////////////////////
	
SX_METHOD_IMPL(abs)
{
	if(args.entries() != 1)
		return false;
	
	r_val = (float)fabs((double)args[0].floatValue());
	return true;
}

SX_METHOD_IMPL(acos)
{
	if(args.entries() != 1)
		return false;
	
	r_val = (float)acos((double)args[0].floatValue());
	return true;
}

SX_METHOD_IMPL(asin)
{
	if(args.entries() != 1)
		return false;
	
	r_val = (float)asin((double)args[0].floatValue());
	return true;
}

SX_METHOD_IMPL(atan)
{
	if(args.entries() != 1)
		return false;
	
	r_val = (float)atan((double)args[0].floatValue());
	return true;
}

SX_METHOD_IMPL(ceil)
{
	if(args.entries() != 1)
		return false;
	
	r_val = (float)ceil((double)args[0].floatValue());
	return true;
}

SX_METHOD_IMPL(cos)
{
	if(args.entries() != 1)
		return false;
	
	r_val = (float)cos((double)args[0].floatValue());
	return true;
}

SX_METHOD_IMPL(cosh)
{
	if(args.entries() != 1)
		return false;
	
	r_val = (float)cosh((double)args[0].floatValue());
	return true;
}

SX_METHOD_IMPL(floor)
{
	if(args.entries() != 1)
		return false;
	
	r_val = (float)floor((double)args[0].floatValue());
	return true;
}

SX_METHOD_IMPL(mod)
{
	if(args.entries() != 2)
		return false;
	
	r_val = (float)fmod((double)args[0].floatValue(), (double)args[1].floatValue());
	return true;
}

SX_METHOD_IMPL(log)
{
	if((args.entries() != 1) || (args[0].floatValue() <= 0))
		return false;
	
	r_val = (float)log((double)args[0].floatValue());
	return true;
}

SX_METHOD_IMPL(sine)
{
	if(args.entries() != 1)
		return false;
	
	r_val = (float)sin((double)args[0].floatValue());
	return true;
}

SX_METHOD_IMPL(sinh)
{
	if(args.entries() != 1)
		return false;
	
	r_val = (float)sinh((double)args[0].floatValue());
	return true;
}

SX_METHOD_IMPL(sqrt)
{
	if(args.entries() != 1)
		return false;
	
	r_val = (float)sqrt((double)args[0].floatValue());
	return true;
}

SX_METHOD_IMPL(tan)
{
	if(args.entries() != 1)
		return false;
	
	r_val = (float)tan((double)args[0].floatValue());
	return true;
}

SX_METHOD_IMPL(tanh)
{
	if(args.entries() != 1)
		return false;
	
	r_val = (float)tanh((double)args[0].floatValue());
	return true;
}
	
	
//////////////////////////////////////////////////////////////////////////////////////////////////////////
// sxMath intialization
//////////////////////////////////////////////////////////////////////////////////////////////////////////
	
SX_INIT_MHT(SXMATH_METHODS);
void SX_IMPL_TYPE::InitMHT()
{
	// hash the methods
	SX_ADD_METHOD(abs);
	SX_ADD_METHOD(acos);
	SX_ADD_METHOD(asin);
	SX_ADD_METHOD(atan);
	SX_ADD_METHOD(ceil);
	SX_ADD_METHOD(cos);
	SX_ADD_METHOD(cosh);
	SX_ADD_METHOD(floor);
	SX_ADD_METHOD(mod);
	SX_ADD_METHOD(log);
	SX_ADD_METHOD(sine);
	SX_ADD_METHOD(sinh);
	SX_ADD_METHOD(sqrt);
	SX_ADD_METHOD(tan);
	SX_ADD_METHOD(tanh);
}
	
} // namespace RFSX
