/****************************************************************************************/
/*	sxMath.h                                                                   	     	*/
/*                                                                                      */
/*  Author: D.R. Gunter                                                                	*/
/*  Description: Class declaration for RFSX Math library.												        */
/*                                                                                      */
/*                                                                                      */
/*  See the accompanying file RealityFactory_Software_License.txt for terms 						*/
/*	on the use of this file. (Under "Reality Factory License")													*/
/*	Software distributed under the License is distributed on an "AS IS"                 */
/*  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See                */
/*  the License for the specific language governing rights and limitations              */
/*  under the License.                                                                  */
/*                                                                                      */
/****************************************************************************************/

#ifndef _SXMATH_H_
#define _SXMATH_H_

#include "RFSX.h"

// sxMath class definition
/*
	
	This class provides access to various standard math functions.
	Should be self-explanatory, as it's just an indirection to the
	standard C math functions. (If someone needs these, they probably
	know how they work.)
	
*/

#ifdef SX_IMPL_TYPE
#undef SX_IMPL_TYPE
#endif
#define SX_IMPL_TYPE sxMath

#define SXMATH_METHODS 15

namespace RFSX
{
	
	class SX_IMPL_TYPE: public skExecutable
	{
		
		public:
			SX_IMPL_TYPE();
			virtual ~SX_IMPL_TYPE();
			static void InitMHT();
			
			virtual bool method(const skString& m_name, skRValueArray& args, skRValue& r_val, skExecutableContext& ctxt);
			virtual int executableType() { return RFSXG_MATH; };
		
		protected:
			SX_METHOD_DECL(abs);
			SX_METHOD_DECL(acos);
			SX_METHOD_DECL(asin);
			SX_METHOD_DECL(atan);
			SX_METHOD_DECL(ceil);
			SX_METHOD_DECL(cos);
			SX_METHOD_DECL(cosh);
			SX_METHOD_DECL(floor);
			SX_METHOD_DECL(mod);
			SX_METHOD_DECL(log);
			SX_METHOD_DECL(sine);
			SX_METHOD_DECL(sinh);
			SX_METHOD_DECL(sqrt);
			SX_METHOD_DECL(tan);
			SX_METHOD_DECL(tanh);
			
		private:
			static sxMHT<SX_IMPL_TYPE> h_method;
		
	};
	
} // namespace RFSX

#undef SX_IMPL_TYPE

#endif // _RFSXMATH_H_
