/****************************************************************************************/
/*	sxVector.cpp                                                                      	*/
/*                                                                                      */
/*  Author: D.R. Gunter                                                                	*/
/*  Description: Implementation for RFSX Vector object																	*/
/*                                                                                      */
/*                                                                                      */
/*  See the accompanying file RealityFactory_Software_License.txt for terms 						*/
/*	on the use of this file. (Under "Reality Factory License")													*/
/*	Software distributed under the License is distributed on an "AS IS"                 */
/*  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See                */
/*  the License for the specific language governing rights and limitations              */
/*  under the License.                                                                  */
/*                                                                                      */
/****************************************************************************************/

#include "sxVector.h"

#ifdef SX_IMPL_TYPE
#undef SX_IMPL_TYPE
#endif
#define SX_IMPL_TYPE sxVector

namespace RFSX
{
		
//////////////////////////////////////////////////////////////////////////////////////////////////////////
// sxVector implementation
//////////////////////////////////////////////////////////////////////////////////////////////////////////
	
SX_IMPL_TYPE::SX_IMPL_TYPE(geFloat x, geFloat y, geFloat z)
{
	vec.X = x;
	vec.Y = y;
	vec.Z = z;
}

SX_IMPL_TYPE::SX_IMPL_TYPE(geVec3d* pvec)
{
	vec = *pvec;
}

SX_IMPL_TYPE::~SX_IMPL_TYPE()
{
	;
}

bool SX_IMPL_TYPE::setValue(const skString& name ,const skString& att, const skRValue& val)
{
	if(name == "x")
	{
		gVec()->X = val.floatValue();
		return true;
	}
	if(name == "y")
	{
		gVec()->Y = val.floatValue();
		return true;
	}
	if(name == "z")
	{
		gVec()->Z = val.floatValue();
		return true;
	}
	return false;
}

bool SX_IMPL_TYPE::getValue(const skString& name ,const skString& att, skRValue& val)
{
	if(name == "x")
	{
		val = (float)this->gVec()->X;
		return true;
	}	
	else if(name == "y")
	{
		val = (float)this->gVec()->Y;
		return true;
	}
	else if(name == "z")
	{
		val = (float)this->gVec()->Z;
		return true;
	}
	else if(name == "mag")
	{
		val = (float)geVec3d_Length(gVec());
		return true;
	}
	else if(name == "length")
	{
		val = (float)geVec3d_Length(gVec());
		return true;
	}
	return skExecutable::getValue(name, att, val);
}

bool SX_IMPL_TYPE::method(const skString& m_name, skRValueArray& args, skRValue& r_val, skExecutableContext& ctxt)
{

	if(h_method(this, m_name, args, r_val, ctxt))
		return true;
	else return skExecutable::method(m_name, args, r_val, ctxt);
	
}
	
	
//////////////////////////////////////////////////////////////////////////////////////////////////////////
// sxVector method functions
//////////////////////////////////////////////////////////////////////////////////////////////////////////
	
SX_METHOD_IMPL(Dot)
{
	sxVector* other = IS_VECTOR(args[0]);
	if(!other)
		return false;
		
	r_val = (float)(geVec3d_DotProduct(caller->gVec(), other->gVec()));
	return true;
}

SX_METHOD_IMPL(Cross)
{
	geVec3d rVec;
	sxVector* other = IS_VECTOR(args[0]);
	if(!other)
		return false;
		
	geVec3d_CrossProduct(caller->gVec(), other->gVec(), &rVec);
	r_val.assignObject(new sxVector(&rVec), true);
	return true;
}

SX_METHOD_IMPL(Normalize)
{
	geVec3d rVec = *(caller->gVec());
	geVec3d_Normalize(&rVec);

	r_val.assignObject(new sxVector(&rVec), true);
	return true;
}

SX_METHOD_IMPL(Scale)
{
	geVec3d rVec;
	
	geVec3d_Scale(caller->gVec(), args[0].floatValue(), &rVec);
	r_val.assignObject(new sxVector(&rVec), true);
	return true;
}

SX_METHOD_IMPL(Subtract)
{
	geVec3d rVec;
	sxVector* other = IS_VECTOR(args[0]);
	if(!other)
		return false;
	
	geVec3d_Subtract(caller->gVec(), other->gVec(), &rVec);
	r_val.assignObject(new sxVector(&rVec), true);
	return true;
}

SX_METHOD_IMPL(Add)
{
	geVec3d rVec;
	sxVector* other = (sxVector*)IS_VECTOR(args[0]);
	if(!other)
		return false;

	geVec3d_Add(caller->gVec(), other->gVec(), &rVec);
	r_val.assignObject(new sxVector(&rVec), true);
	return true;
}

SX_METHOD_IMPL(Inverse)
{
	geVec3d rVec = *(caller->gVec());

	geVec3d_Inverse(&rVec);
	r_val.assignObject(new sxVector(&rVec), true);
	return true;
}

SX_METHOD_IMPL(DistanceTo)
{
	sxVector* other = IS_VECTOR(args[0]);
	if(!other)
		return false;
	
	r_val = (float)geVec3d_DistanceBetween(caller->gVec(), other->gVec());
	return true;
}
	
	
//////////////////////////////////////////////////////////////////////////////////////////////////////////
// sxVector initialization
//////////////////////////////////////////////////////////////////////////////////////////////////////////

SX_INIT_MHT(SXVECTOR_METHODS);
void SX_IMPL_TYPE::InitMHT()
{
		
	SX_ADD_METHOD(Dot);
	SX_ADD_METHOD(Cross);
	SX_ADD_METHOD(Normalize);
	SX_ADD_METHOD(Scale);
	SX_ADD_METHOD(Subtract);
	SX_ADD_METHOD(Add);
	SX_ADD_METHOD(Inverse);
	SX_ADD_METHOD(DistanceTo);	
	
}
	
} // namespace RFSX

#undef SX_IMPL_TYPE
