/****************************************************************************************/
/*	sxVector.h                                                                   	      */
/*                                                                                      */
/*  Author: D.R. Gunter                                                                	*/
/*  Description: Class declaration for RFSX vector object.											        */
/*                                                                                      */
/*                                                                                      */
/*  See the accompanying file RealityFactory_Software_License.txt for terms 						*/
/*	on the use of this file. (Under "Reality Factory License")													*/
/*	Software distributed under the License is distributed on an "AS IS"                 */
/*  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See                */
/*  the License for the specific language governing rights and limitations              */
/*  under the License.                                                                  */
/*                                                                                      */
/****************************************************************************************/

#ifndef _SX_VECTOR_H_
#define _SX_VECTOR_H_

#include "RFSX.h"

#include <vec3D.h>

// sxVector class definition
/*

	This class provides a wrapper for a geVec3D struct and provides
	an interface to Genesis3D vector manipulation functions.
	
	Fields(R/W):
		x -- vector x-axis value
		y -- vector y-axis value
		z -- vector z-axis value
		
	Fields(RO):
		mag -- current "calculated" vector magnitude
		length -- alias for m
		
	Methods:
		
		
		Note: Vector methods never change the data of the calling vector.
			Instead, they return a new sxVector with the result of the operation.
	
*/

#ifdef SX_IMPL_TYPE
#undef SX_IMPL_TYPE
#endif
#define SX_IMPL_TYPE sxVector
#define SXVECTOR_METHODS 8

// Type-check and upcast macro
#define IS_VECTOR(o) CheckType<SX_IMPL_TYPE>(o, RFSXU_VECTOR);

namespace RFSX
{
	
	class sxVector : public skExecutable
	{
		
		public:
			SX_IMPL_TYPE(geFloat x, geFloat y, geFloat z);
			SX_IMPL_TYPE(geVec3d* vec);
			virtual ~SX_IMPL_TYPE();
			static void InitMHT();
			
			virtual bool setValue(const skString& name ,const skString& att, const skRValue& val);
			virtual bool getValue(const skString& name ,const skString& att, skRValue& val);
			virtual bool method(const skString& m_name, skRValueArray& args, skRValue& r_val, skExecutableContext& ctxt);
			virtual int executableType() { return RFSXU_VECTOR; };
			
			geVec3d* gVec() { return &vec; };
		
		protected:
			SX_METHOD_DECL(Dot);
			SX_METHOD_DECL(Cross);
			SX_METHOD_DECL(Normalize);
			SX_METHOD_DECL(Scale);
			SX_METHOD_DECL(Subtract);
			SX_METHOD_DECL(Add);
			SX_METHOD_DECL(Inverse);
			SX_METHOD_DECL(DistanceTo);
		
		private:
			geVec3d vec;
			
			static sxMHT<SX_IMPL_TYPE> h_method;
		
	};
	
} // namespace RFSX

#undef SX_IMPL_TYPE

#endif // _RFSX_VECTOR_H_
